@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2010 Olaf Till
+Copyright (C) 2010-2016 Olaf Till
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@ along with this program. If not, see <http://www.gnu.org/licenses/>.
 #include <octave/oct.h>
 #include <octave/ov-struct.h>
 
+#include "error-helpers.h"
+
 DEFUN_DLD (cell2fields, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Loadable Function} {} cell2fields (@var{c}, @var{fields}, @var{dim}, @var{s})\n\
@@ -28,25 +30,22 @@ Return structure-array @var{s} after setting the fields @var{fields} with the sl
 {
   std::string fname ("cell2fields");
 
+  octave_value_list err_retval;
+
   if (args.length () != 4)
     {
       print_usage ();
-      return octave_value_list ();
+      return err_retval;
     }
 
-  Cell c = args(0).cell_value ();
-  if (error_state)
-    {
-      error ("%s: first argument must be a cell array", fname.c_str ());
-      return octave_value_list ();
-    }
+  Cell c;
+  CHECK_ERROR (c = args(0).cell_value (), err_retval,
+               "%s: first argument must be a cell array", fname.c_str ());
 
-  Array<std::string> names = args(1).cellstr_value ();
-  if (error_state)
-    {
-      error ("%s: second argument must be a cell array of strings", fname.c_str ());
-      return octave_value_list ();
-    }
+  Array<std::string> names;
+  CHECK_ERROR (names = args(1).cellstr_value (), err_retval,
+               "%s: second argument must be a cell array of strings",
+               fname.c_str ());
 
   // do not reject if ndims == [0, 0]
   dim_vector ndims = names.dims ();
@@ -54,23 +53,16 @@ Return structure-array @var{s} after setting the fields @var{fields} with the sl
     {
       error ("%s: second argument must be a one-dimensional cell array",
              fname.c_str ());
-      return octave_value_list ();
+      return err_retval;
     }
 
-  octave_idx_type dim = args(2).int_value ();
-  if (error_state)
-    {
-      error ("%s: third argument must be an integer",
-             fname.c_str ());
-      return octave_value_list ();
-    }
+  octave_idx_type dim;
+  CHECK_ERROR (dim = args(2).int_value (), err_retval,
+               "%s: third argument must be an integer", fname.c_str ());
 
-  octave_map s = args(3).map_value ();
-  if (error_state)
-    {
-      error ("%s: fourth argument must be a structure", fname.c_str ());
-      return octave_value_list ();
-    }
+  octave_map s;
+  CHECK_ERROR (s = args(3).map_value (), err_retval,
+               "%s: fourth argument must be a structure", fname.c_str ());
 
   octave_idx_type i, j;
 
@@ -95,12 +87,12 @@ Return structure-array @var{s} after setting the fields @var{fields} with the sl
     {
       error ("%s: second argument has incorrect length", fname.c_str ());
 
-      return octave_value_list ();
+      return err_retval;
     }
 
   octave_map retval;
 
-  if (s.keys (). length () == 0)
+  if (s.keys (). numel () == 0)
     retval.resize (tdims);
   else
     {
@@ -108,7 +100,7 @@ Return structure-array @var{s} after setting the fields @var{fields} with the sl
         {
           error ("%s: structure has incorrect dimensions", fname.c_str ());
 
-          return octave_value_list ();
+          return err_retval;
         }
       retval = (s);
     }
